from alpha_vantage.timeseries import TimeSeries
import sqlite3


class Stock:
    def __init__(self, ticker, key=''):
        self.symbol = ticker.upper()
        if key == '':
            self.key = 'WTTQPLFVUWLDDLB'
        else:
            self.key = key

    def update(self, outputsize='compact', export_to_db=False, filename='stockdb.sqlite3', path=''):

        ts = TimeSeries(self.key, output_format='pandas')

        data, meta_data = ts.get_daily(self.symbol, outputsize)
        data.rename(columns={'1. open': 'open', '2. high': 'high', '3. low': 'low',
                             '4. close': 'close', '5. volume': 'volume'}, inplace=True)

        def export():
            conn = sqlite3.connect(path + filename)
            cur = conn.cursor()

            cur.executescript('''
                        CREATE TABLE IF NOT EXISTS
                        stock_data (
                        stock_id INTEGER, date TEXT, open NUMBER, high NUMBER, low NUMBER, close NUMBER, volume NUMBER
                        );

                        CREATE TABLE IF NOT EXISTS stock_index (
                        id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, symbol TEXT UNIQUE
                        );
                        ''')

            cur.execute('INSERT OR IGNORE INTO stock_index (symbol) VALUES ( ? )', (self.symbol,))
            cur.execute('SELECT id FROM stock_index WHERE symbol = ?', (self.symbol,))
            stock_id = cur.fetchone()[0]

            data.insert(loc=0, column='stock_id', value=stock_id)
            data.to_sql('stock_data', conn, if_exists='append', index=True, index_label='date')

            cur.execute('''DELETE FROM stock_data
                            WHERE ROWID not in(
                                SELECT min(ROWID)
                                FROM stock_data
                                GROUP BY stock_id, date
                            );''')

            conn.commit()
            cur.close()

        if export_to_db is True:
            export()
        else:
            return data



from setuptools import setup

setup(name='quantly',
      version='0.1',
      description='A multi-tool financial suite for equity traders.',
      url='https://github.com/BluechipData/quantly',
      author='Brandon Boisvert',
      license='MIT',
      packages=['quantly'],
      install_requires=['pandas', 'alphavantage', 'pystan', 'fbprophet'],
      zip_safe=False)



import pandas as pd
import sqlite3
from fbprophet import Prophet
import matplotlib.pyplot as plt


def from_database(path: str, table_name: str, selection: str, date_column: str, data_column: str,
                  direction: str = 'ascending', limit: int = 0, d_seasonality: bool = True, y_seasonality: bool = True,
                  forecast_periods: int = 365, chart: bool = True):

    conn = sqlite3.connect(path)

    if direction == 'ascending':
        direction = ' ASC '
    else:
        direction = ' DSC '

    if limit == 0:
        query = 'SELECT ' + selection + ' FROM ' + table_name + direction
    else:
        query = 'SELECT ' + selection + ' FROM ' + table_name + direction + ' LIMIT ' + str(limit)

    df = pd.read_sql_query(query, conn)
    df = df[[date_column, data_column]]
    df.rename(columns={date_column: 'ds', data_column: 'y'}, inplace=True)

    m = Prophet(daily_seasonality=d_seasonality, yearly_seasonality=y_seasonality)
    m.fit(df)

    future = m.make_future_dataframe(periods=forecast_periods)

    forecast = m.predict(future)

    if chart is True:
        fig1 = m.plot(forecast)
        plt.show(fig1)
        return forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']]
    else:
        return forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']]


def from_dataframe(dataframe: vars, date_column: str, data_column: str, d_seasonality: bool = True,
                   y_seasonality: bool = True, forecast_periods: int = 365, chart: bool = True):

    df = dataframe[[date_column, data_column]]
    df.rename(columns={date_column: 'ds', data_column: 'y'}, inplace=True)

    m = Prophet(daily_seasonality=d_seasonality, yearly_seasonality=y_seasonality)
    m.fit(df)

    future = m.make_future_dataframe(periods=forecast_periods)

    forecast = m.predict(future)

    if chart is True:
        fig1 = m.plot(forecast)
        plt.show(fig1)
        return forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']]
    else:
        return forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']]


def from_csv(path: str, date_column: str, data_column: str, sep: str = ',', d_seasonality: bool = True,
             y_seasonality: bool = True, forecast_periods: int = 365, chart: bool = True):

    df = pd.read_csv(path, sep)
    df.rename(columns={date_column: 'ds', data_column: 'y'}, inplace=True)

    m = Prophet(daily_seasonality=d_seasonality, yearly_seasonality=y_seasonality)
    m.fit(df)

    future = m.make_future_dataframe(periods=forecast_periods)

    forecast = m.predict(future)

    if chart is True:
        fig1 = m.plot(forecast)
        plt.show(fig1)
        return forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']]
    else:
        return forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']]
